@@ -19,6 +19,8 @@ SP             <- function(x, ...) UseMethod("SP")
 Throughputc    <- function(x, ...) UseMethod("Throughputc")
 Throughputk    <- function(x, ...) UseMethod("Throughputk")
 Throughputck   <- function(x, ...) UseMethod("Throughputck")
+Throughputn    <- function(x, ...) UseMethod("Throughputn")
+Throughputcn   <- function(x, ...) UseMethod("Throughputcn")
 Lc             <- function(x, ...) UseMethod("Lc")
 Lk             <- function(x, ...) UseMethod("Lk")
 Lck            <- function(x, ...) UseMethod("Lck")
@@ -40,18 +42,7 @@ is.anomalous <- function(x)
 }
 
 
-logFact <- function(n)
-{
-  if (n==0 || n==1) 0
-  else 
-    #(0.5 * log(2 * pi * n)) + (n * log(n/exp(1))) + 
-    #log(1 + 1/(12*n) + 1/(288*(n^2)) - 139/(51840 * (n^3)) - 571/(2488320 * (n^4)))
-    (0.5 * log(2 * pi * n)) + (n * log(n/exp(1))) + 
-    log(exp(1/(12 * n)) - exp(1/(360 * (n^3))) + exp(1/(1260 * (n^5))) - exp(1/(1680 * (n^7))) + exp(1/(1188 * (n^9)))
-    - exp(1/(360360 * (n^11))) + exp(1/(156 * (n^13))) - exp(1/(122400 * (n^15))) + exp(1/(244180 * (n^17)))
-    - exp(1/(125400 * (n^19))) + exp(1/(5796 * (n^21))) - exp(1/(1506960 * (n^23))) + exp(1/(300 * (n^25))))
-}
-
+is.wholenumber <- function(x, tol = .Machine$double.eps^0.5)  abs(x - round(x)) < tol
 
 C_erlang2 <- function(c, r)
 {
@@ -163,7 +154,7 @@ B_erlang3 <- function(c, u)
 }
 
 
-B_erlang <- function(c=1, u=0)
+B_erlang4 <- function(c=1, u=0)
 {
 
   if (is.anomalous(c))
@@ -190,6 +181,59 @@ B_erlang <- function(c=1, u=0)
 	
 }
 
+B_erlang5 <- function(c=1, u=0)
+{
+
+  if (is.anomalous(c))
+    stop("The parameter c is anomalous. Check it!")
+
+  if (is.anomalous(u))
+    stop("The parameter u is anomalous. Check it!")
+
+  if (c<0)
+    stop("The number of servers can not be less than zero!")
+
+  if (u<0)
+    stop("The u parameter can not be negative!")
+
+  if (!is.wholenumber(c))
+    stop("The parameter c has to be an integer number")
+
+  1/Reduce(function(i, j) 1 + (i * j), (1 / u) * seq(1, c), init=1)    
+}
+
+
+B_erlang <- function(c=1, u=0)
+{
+
+  if (is.anomalous(c))
+    stop("The parameter c is anomalous. Check it!")
+
+  if (is.anomalous(u))
+    stop("The parameter u is anomalous. Check it!")
+
+  if (c<0)
+    stop("The number of servers can not be less than zero!")
+
+  if (u<0)
+    stop("The u parameter can not be negative!")
+
+  if (!is.wholenumber(c))
+    stop("The parameter c has to be an integer number")
+
+  if (c==0) return(1)
+  if (u==0) return(0)
+
+  val <- 1/u
+  tot <- 1
+  
+  for (i in 1:c)
+    tot <- 1 + (tot * i * val)
+
+  1/tot    
+}
+
+
 ProbFactCalculus <- function(lambda, mu, c, k, m, limit, fAuxC, fAuxK, fAuxM)
 {
   pn <- c(0:limit)
@@ -261,7 +305,7 @@ CheckInput.i_MM1 <- function(x, ...)
  if (x$lambda < 0)
 	stop(MM1_lambda_zpositive)
 
- if (x$n != floor(x$n))
+ if (!is.wholenumber(x$n))
   stop(MM1_n_integer)
 
  ro <- x$lambda / x$mu
@@ -303,7 +347,7 @@ QueueingModel.i_MM1 <- function(x, ...)
 
 
   res <- list(
-    Inputs = x, RO = RO, Lq = Lq, Wq = Wq, Throughput = Throughput, L = L, W = W, WWq = Wq, LLq = LLq,
+    Inputs = x, RO = RO, Lq = Lq, Wq = Wq, Throughput = Throughput, L = L, W = W, WWq = W, LLq = LLq,
     Pn = Pn, FW = FW, FWq = FWq 
   )
 
@@ -366,6 +410,7 @@ CheckInput.i_MMC <- function(x, ...)
   MMC_n_integer <- "the number of clients must be a integer number"
   MMC_anomalous <- "Some value of lambda, mu, c or n is anomalous. Check the values."
   MMC_method <- "method variable has to be 0 to be exact calculus, 1 to be aproximate calculus"
+  MMC_c_integer <- "the number of servers (c) must be a integer number"
 
 
   if (class(x) != "i_MMC")
@@ -395,7 +440,10 @@ CheckInput.i_MMC <- function(x, ...)
     stop(MMC_r_c_warning)
   }
 
-  if (x$n != floor(x$n))
+  if (!is.wholenumber(x$c))
+		stop(MMC_c_integer)
+
+  if (!is.wholenumber(x$n))
 		stop(MMC_n_integer)
 
   if (x$method != 0 && x$method != 1)
@@ -770,7 +818,7 @@ CheckInput.i_MMCKK <- function(x, ...)
  MMCKK_class <- "The class of the object x has to be M/M/c/K/K (i_MMCKK)"
  MMCKK_anomalous <- "Some value of lambda, mu, c or k is anomalous. Check the values."
  MMCKK_method <- "method variable has to be 0 to be exact calculus, 1 to be aproximate calculus"
-
+ MMCKK_c_integer <- "c must be integer"
 
  if (class(x) != "i_MMCKK")
   stop(MMCKK_class)
@@ -789,6 +837,9 @@ CheckInput.i_MMCKK <- function(x, ...)
  if (x$c < 1)
  	stop(MMCKK_c_one)
 
+ if (!is.wholenumber(x$c))
+   stop(MMCKK_c_integer)
+
  if (x$k < 1)
  	stop(MMCKK_k_one)
 
@@ -1124,6 +1175,7 @@ CheckInput.i_MMCKM <- function(x, ...)
  MMCKM_class <- "The class of the object x has to be M/M/c/K/m (i_MMCKM)"
  MMCKM_anomalous <- "Some value of lambda, mu, c, k or m is anomalous. Check the values."
  MMCKM_method <- "method variable has to be 0 to be exact calculus, 1 to be aproximate calculus"
+ MMCKM_c_integer <- "c must be integer"
 
  if (class(x) != "i_MMCKM")
   stop(MMCKM_class)
@@ -1134,25 +1186,28 @@ CheckInput.i_MMCKM <- function(x, ...)
     stop(MMCKM_anomalous)
 
  if (x$lambda < 0)
-	stop(MMCKM_lambda_zpositive)
+	 stop(MMCKM_lambda_zpositive)
 
  if (x$mu <= 0)
- 	stop(MMCKM_mu_positive)
+ 	 stop(MMCKM_mu_positive)
 
  if (x$c < 1)
- 	stop(MMCKM_c_one)
+ 	 stop(MMCKM_c_one)
+
+ if (!is.wholenumber(x$c))
+   stop(MMCKM_c_integer)
 
  if (x$k < 1)
- 	stop(MMCKM_k_one)
+ 	 stop(MMCKM_k_one)
 
  if (x$m < 1)
- 	stop(MMCKM_m_one)
+ 	 stop(MMCKM_m_one)
 
  if (x$k < x$c)
-	stop(MMCKM_k_c)
+	 stop(MMCKM_k_c)
  
  if (x$m < x$k)
-	stop(MMCKM_m_k)
+	 stop(MMCKM_m_k)
 
  if (x$method != 0 && x$method != 1)
    stop(MMCKM_method)
@@ -1479,7 +1534,7 @@ CheckInput.i_MMInf <- function(x, ...)
  	if (x$lambda < 0)
 		stop(MMInf_lambda_zpositive)
 
-  if (x$n != floor(x$n))
+  if (!is.wholenumber(x$n))
     stop(MMInf_n_integer)
 }
 
@@ -1505,7 +1560,7 @@ QueueingModel.i_MMInf <- function(x, ...)
 
   # The result
   res <- list(
-    Inputs=x, RO = 0, Lq = 0, Wq = 0, Throughput = Throughput, L = L, W = W, LLq = 0, WWq = 0,
+    Inputs=x, RO = L, Lq = 0, Wq = 0, Throughput = Throughput, L = L, W = W, LLq = 0, WWq = 0,
     Pn = Pn, FW = FW, FWq = FWq
   )
 
@@ -1711,6 +1766,7 @@ NewInput.MMCK <- function(lambda=0, mu=0, c=1, k=1)
 CheckInput.i_MMCK <- function(x, ...)
 {
  MMCK_c_one <- "c has to be at least one!!"
+ MMCK_c_integer <- "c has to be integer!!"
  MMCK_lambda_zpositive <- "lambda must be equal or greater than zero"
  MMCK_k_one <- "k has to be at least one!!"
  MMCK_k_c <- "k must be equal or greater than the number of servers c"
@@ -1735,6 +1791,9 @@ CheckInput.i_MMCK <- function(x, ...)
  if (x$c < 1)
 	stop(MMCK_c_one)
 
+ if (!is.wholenumber(x$c))
+   stop(MMCK_c_integer)
+
  if (x$k < 1)
 	stop(MMCK_k_one)
 
@@ -1905,6 +1964,7 @@ CheckInput.i_MMCC <- function(x, ...)
  MMCC_mu_positive <- "mu must be greater than zero"
  MMCC_class <- "the class of the object x has to be M/M/C/C (i_MMCC)"
  MMCC_anomalous <- "Some value of lambda, mu or c is anomalous. Check the values."
+ MMCC_c_integer <- "c has to be integer!!"
 
  if (class(x) != "i_MMCC")
    stop(MMCC_class)
@@ -1920,6 +1980,9 @@ CheckInput.i_MMCC <- function(x, ...)
 
  if (x$c < 1)
 	stop(MMCC_c_one)
+
+ if (!is.wholenumber(x$c))
+   stop(MMCC_c_integer)
 }
 
 
@@ -2163,6 +2226,8 @@ CheckInput.i_OJN <- function(x, ...)
    if (!is_prob_a_matrix && (x$nodes[[i]]$lambda != x$nodes[[1]]$lambda))
      stop(all_lambda_equals)
 
+   CheckInput(n)
+
    i <- i + 1
  }
 
@@ -2216,30 +2281,30 @@ Pn.o_OJN <- function(x, ...) { x$Pn }
 summary.o_OJN <- function(object, ...)
 {
    
-  print("The inputs of the open Jackson network are:")
+  cat("The inputs of the open Jackson network are:\n\n")
   print(object$Inputs)
-  print("", quote=FALSE)
-  print("The outputs of the open Jackson network are:")
-  print("", quote=FALSE)
 
-  print("---------- Complete network -------------------------")
-  print(paste("The mean number of clients in the network is: ", object$L))
-  print(paste("The mean time spend in the network is: ", object$W))
-  print(paste("The throughput of the network is: ", object$Throughput))
-  print("", quote=FALSE)
+  cat("\n\n")
+  cat(paste("The outputs of the open Jackson network are:", "\n\n", sep=""))
 
-  print("--------- Per node ---------------------------------")
+  cat("---------- Complete network -------------------------\n\n")
+  cat(paste("The mean number of clients in the network is: ", object$L, "\n", sep=""))
+  cat(paste("The mean time spend in the network is: ", object$W, "\n", sep=""))
+  cat(paste("The throughput of the network is: ", object$Throughput, "\n", sep=""))
+  cat("\n\n")
+
+  cat("--------- Per node ---------------------------------\n\n")
  
   i <- 1
   while (i <= length(object$ROk))
   {
-    print(paste("The use of node ", i, " is: ", object$ROk[i]))
-    print(paste("The throughput of node ", i, " is: ", object$Throughputk[i]))
-    print(paste("The mean number of clients in node ", i, " is: ", object$Lk[i]))
-    print(paste("The mean time spend in node ", i, " is: ", object$Wk[i]))
-    print(paste("The probability (p0, p1, ..., pn) or visit ratio of node ", i, " is: "))
+    cat(paste("The use of node ", i, " is: ", object$ROk[i], "\n", sep=""))
+    cat(paste("The throughput of node ", i, " is: ", object$Throughputk[i], "\n", sep=""))
+    cat(paste("The mean number of clients in node ", i, " is: ", object$Lk[i], "\n", sep=""))
+    cat(paste("The mean time spend in node ", i, " is: ", object$Wk[i], "\n", sep=""))
+    cat(paste("The probability (p0, p1, ..., pn) or visit ratio of node ", i, " is: ", "\n", sep=""))
     print(object$Pn[[i]])
-    print("", quote=FALSE)
+    cat("\n\n")
     i <- i + 1
   }
 }
@@ -2248,9 +2313,9 @@ summary.o_OJN <- function(object, ...)
 ## Closed Jackson Network
 #######################################################################################
 
-NewInput.CJN <- function(prob=NULL, n=0, z=0, operational=FALSE, ...)
+NewInput.CJN <- function(prob=NULL, n=0, z=0, operational=FALSE, method=0, tol=0.001,...)
 {
-  nds <- list(prob=prob, n=n, z=z, operational=operational, nodes=nodes(...))
+  nds <- list(prob=prob, n=n, z=z, operational=operational, method=method, tol=tol, nodes=nodes(...))
   class(nds) <- "i_CJN"
   nds
 }
@@ -2264,8 +2329,14 @@ CheckInput.i_CJN <- function(x, ...)
  row_distinct_nodes <- "x$prob (matrix class) has distinct number of rows that the number of nodes in x$nodes."
  visit_ratios_wrong <- "x$prob contains a different number of visit ratios than x$nodes."
  n_greater_zero <- "n has to be greater than zero"
+ CJN_operational_logical <- "The x$operational parameter has to be of class logical (TRUE or FALSE)"
+ CJN_method_values <- "The x$method has to be 0 (exact) or 1 (aprox)"
+ CJN_tol_value <- "The x$tol has to be positive"
 
- if (is.anomalous(x$prob) || is.anomalous(x$nodes) || is.anomalous(x$n) || is.anomalous(x$z))
+ if (
+   is.anomalous(x$prob) || is.anomalous(x$nodes) || is.anomalous(x$n) ||
+   is.anomalous(x$z) || is.anomalous(x$operational)
+ )
     stop(x_anomalous)
 
  if (class(x) != "i_CJN")
@@ -2276,6 +2347,12 @@ CheckInput.i_CJN <- function(x, ...)
 
  num_nodes <- length(x$nodes)
 
+ if (x$method != 0 && x$method != 1)
+   stop(CJN_method_values)
+
+ if (!(x$tol > 0))
+   stop(CJN_tol_value)
+
  is_prob_a_matrix <- (class(x$prob) == "matrix")
 
  if (is_prob_a_matrix)
@@ -2292,27 +2369,49 @@ CheckInput.i_CJN <- function(x, ...)
      stop(visit_ratios_wrong)
  }
 
+ if (class(x$operational) != "logical")
+   stop(CJN_operational_logical) 
+
  i <- 1
  while (i <= num_nodes)
  {
    n = x$nodes[[i]]
 
-   if (class(n) != "i_MM1" && class(n) != "i_MMC" && class(n) != "i_MMInf")
-     stop(paste(paste("Node ", i), "is not of class i_MM1 or i_MMC or i_MMInf!!"))
+   if (x$method == 0)
+   {
+     if (class(n) != "i_MM1" && class(n) != "i_MMC" && class(n) != "i_MMInf")
+       stop(paste(paste("Node ", i), "is not of class i_MM1 or i_MMC or i_MMInf!!"))
+   }
+   else
+   {
+     if (class(n) != "i_MM1" && class(n) != "i_MMInf")
+       stop(paste(paste("Node ", i), "is not of class i_MM1 or i_MMInf!!"))
+   }
+
+   CheckInput(n)
      
    i <- i + 1
  }
-
 }
 
 
 QueueingModel.i_CJN <- function(x, ...)
 {
-
   CheckInput(x)
 
+  if (x$method == 0)
+    QueueingModelExact(x, ...)
+  else
+    QueueingModelApprox(x, ...)  
+}
+
+
+QueueingModelExact <- function(x, ...)
+{
   num_nodes <- length(x$nodes)
 
+  Throughputn <- rep(0, x$n)
+
   if (class(x$prob) == "matrix")
   {
     ident <- diag(dim(x$prob)[1])
@@ -2449,6 +2548,9 @@ QueueingModel.i_CJN <- function(x, ...)
 
     Throughput <- i / tmp
     #print(paste("throughput: ", throughput))
+    
+    # The vector with the n values is updated
+    Throughputn[i] <- Throughput
 
     k <- 1
     while (k <= num_nodes)
@@ -2469,7 +2571,7 @@ QueueingModel.i_CJN <- function(x, ...)
   while (k <= num_nodes)
   {
     if (class(x$nodes[[k]]) == "i_MMInf")
-      ROk[k] <- 0
+      ROk[k] <- Lk[k]
     else if (class(x$nodes[[k]]) == "i_MM1")
       ROk[k] <- Throughputk[k] * (1/x$nodes[[k]]$mu)
     else #class i_MMC  
@@ -2498,7 +2600,8 @@ QueueingModel.i_CJN <- function(x, ...)
       ROk = ROk,
       Throughputk = Throughputk,
       Wk = Wk,
-      Lk = Lk
+      Lk = Lk,
+      Throughputn = Throughputn
     )
   
   class(res) <- "o_CJN"
@@ -2506,6 +2609,113 @@ QueueingModel.i_CJN <- function(x, ...)
 }
 
 
+QueueingModelApprox <- function(x, ...)
+{
+  num_nodes <- length(x$nodes)
+
+  Throughputn <- numeric()
+
+  if (class(x$prob) == "matrix")
+  {
+    ident <- diag(dim(x$prob)[1])
+    const <- matrix(data=1, nrow=dim(x$prob)[1], ncol=1)
+    all1 <- matrix(data=1, ncol=dim(x$prob)[2], nrow=dim(x$prob)[1])
+    prob_est <- t(solve(t(x$prob + all1 - ident), const))
+  }
+  else
+  {
+    if (x$operational) #Visit ratios as counts of repetitions has been given
+    {
+      prob_est <- rep(1, num_nodes)
+      #We have to "correct" the mu values
+      for (i in (1:num_nodes))
+        x$nodes[[i]]$mu <- x$nodes[[i]]$mu / x$prob[i]     
+    }
+    else
+      prob_est <- x$prob
+  }
+  
+  # array initialization
+  Wk <- rep(0, num_nodes)
+  Lk <- rep(x$n / num_nodes, num_nodes)
+  Throughputk <- rep(0, num_nodes)
+  Throughput <- 0
+  
+  finIter <- FALSE
+  numIter <- 1
+
+  while (!finIter)
+  {
+    Arrk <- ((x$n-1)/x$n) * Lk
+  
+    acum <- 0
+    for (i in (1:num_nodes))
+    {
+      if (class(x$nodes[[i]]) == "i_MMInf")
+        Wk[i] <- 1/x$nodes[[i]]$mu
+      else
+        Wk[i] <- 1/x$nodes[[i]]$mu * (1 + Arrk[i])
+
+      acum <- acum + Wk[i] 
+    }
+
+    Throughput <- x$n / (x$z + acum)
+    Throughputn[numIter] <- Throughput
+
+    LkAux <- Throughput * Wk
+
+    if (sum(abs((LkAux - Lk)) < x$tol) == num_nodes)
+      finIter <- TRUE
+    else
+      {
+        Lk <- LkAux
+        numIter <- numIter + 1
+      }
+    }
+  
+  Throughputk <- prob_est * Throughput
+  
+  ROk <- rep(0, num_nodes)
+
+  for (i in (1:num_nodes))
+  {
+    if (class(x$nodes[[i]]) == "i_MMInf")
+      ROk[i] <- Lk[i] 
+    else
+      ROk[i] <- Throughputk[i] * (1/x$nodes[[i]]$mu)
+  }
+
+  W <- (x$n / Throughput) - x$z
+  L <- x$n - (Throughput * x$z)
+
+  if (x$operational)
+  {
+    for (i in (1:num_nodes))
+    {
+      x$nodes[[i]]$mu <- x$nodes[[i]]$mu * x$prob[i]
+      Throughputk[i] <- Throughputk[i] * x$prob[i]
+    }
+  }
+
+  res <-
+    list(
+      Inputs = x,
+      Throughput = Throughput,
+      L = L,
+      W = W,
+      ROk = ROk,
+      Throughputk = Throughputk,
+      Wk = Wk,
+      Lk = Lk,
+      Throughputn = Throughputn
+    )
+  
+  class(res) <- "o_CJN"
+  res
+}
+
+
+
 Inputs.o_CJN <- function(x, ...) { x$Inputs }
 Throughput.o_CJN <- function(x, ...) { x$Throughput }
 L.o_CJN <- function(x, ...) { x$L }
@@ -2514,32 +2724,32 @@ ROk.o_CJN <- function(x, ...) { x$ROk }
 Throughputk.o_CJN <- function(x, ...) { x$Throughputk }
 Lk.o_CJN <- function(x, ...) { x$Lk }
 Wk.o_CJN <- function(x, ...) { x$Wk }
-
+Throughputn.o_CJN <- function(x, ...) { x$Throughputn }
 
 summary.o_CJN <- function(object, ...)
 {
    
-  print("The inputs of the closed Jackson network are:")
+  cat("The inputs of the closed Jackson network are:\n\n")
   print(object$Inputs)
-  print("", quote=FALSE)
-  print("The outputs of the closed Jackson network are:")
-  print("", quote=FALSE)
+  cat("\n\n")
+  cat("The outputs of the closed Jackson network are:\n\n")
   
-  print("---------- Complete network -------------------------")
-  print(paste("The mean number of clients in the network is: ", object$L))
-  print(paste("The mean time spend in the network is: ", object$W))  
-  print("", quote=FALSE)
+  cat("---------- Complete network -------------------------\n\n")
+  cat(paste("The mean number of clients in the network is: ", object$L, "\n", sep=""))
+  cat(paste("The mean time spend in the network is: ", object$W, "\n", sep=""))  
+  cat("\n\n")
 
 
-  print("--------- Per node ---------------------------------")
+  cat("--------- Per node ---------------------------------\n\n")
  
   i <- 1
   while (i <= length(object$ROk))
   {
-    print(paste("The use of node ", i, " is: ", object$ROk[i]))
-    print(paste("The mean number of clients in node ", i, " is: ", object$Lk[i]))
-    print(paste("The response time in node ", i, " is: ", object$Wk[i]))
-    print(paste("The throughput of node ", i, " is: ", object$Throughputk[i]))
+    cat(paste("The use of node ", i, " is: ", object$ROk[i], "\n", sep=""))
+    cat(paste("The mean number of clients in node ", i, " is: ", object$Lk[i], "\n", sep=""))
+    cat(paste("The response time in node ", i, " is: ", object$Wk[i], "\n", sep=""))
+    cat(paste("The throughput of node ", i, " is: ", object$Throughputk[i], "\n", sep=""))
+    cat("\n\n")
     i <- i + 1
   }
 }
@@ -2843,9 +3053,8 @@ CheckInput.i_MCCN <- function(x, ...)
 }
 
 QueueingModel.i_MCCN <- function(x, ...)
+#check <- function(x, ...)
 {
-  #print("Comienza la rutina")  
-
   CheckInput(x)
 
   Throughputc <- rep(0, x$classes)
@@ -2859,82 +3068,85 @@ QueueingModel.i_MCCN <- function(x, ...)
   Throughputk <- rep(0, x$nodes)
   Wk <- rep(0, x$nodes)
 
-  #print("Despues de inicialización")
-
   LkAux <- list()
 
   # from the last to the first of x$vNumber
   i <- x$classes
   while (i > 0)
   {
-    #print("Dentro del while")
     LkAux <- c(LkAux, list(seq(1, x$vNumber[i] + 1)))  
     i <- i - 1
   }
 
-  #print("La estructura dinámica ha sido creada")
-  #print(paste("LkAux: ", LkAux))
-
   # Generate the matrix of elements to iterate
   # the elements has to be reversed when iterate over it
   iter <- as.matrix(expand.grid(LkAux))
-  numInter <- dim(iter)[1]
-  #print(paste("numInter: ", numInter))
+  
+  numRowsIter <- nrow(iter)
+
+  orderCol <- rep(0, numRowsIter)
+
+  # calculate the order num of each row
+  for (i in 1:numRowsIter)
+    orderCol[i] <- sum(iter[i, ])
+
+  #attach it to the iter matrix
+  iter <- cbind(iter, orderCol)
+
+  # Order ir by the OrderCol column
+  iter <- iter[order(orderCol), ]
+
+  #orderCol has to be replaced to the column already ordered
+  orderCol <- iter[, ncol(iter)]
 
   LkV <- array(0, dim=c(x$vNumber + 1, x$nodes))
+  Throughputcn <- array(0, dim=c(x$vNumber + 1, x$classes))
   
-  #print("LkV: ")
-  #print(LkV)
-
-  for (n in (2:numInter))
+  for (n in (3:(sum(x$vNumber+1))))
   {
-    elem <- as.vector(rev(iter[n, ]))
-    #print("elem: ")
-    #print(elem)
+    
+    #Get the population to iterate
+    population <- iter[orderCol == n, ]
 
-    for (cla in (1:x$classes))
+    cardin <- nrow(population)
+    if (is.null(cardin))
+      cardin <- 1
+
+    #Now, getting each elem
+    for (subpopulation in 1:cardin)
     {
-      vaux <- elem
-      vaux[cla] <- max(elem[cla]-1, 1)
-      #print("vaux: ")
-      #print(vaux)
+      if (cardin == 1)
+        elem <- as.vector(population)
+      else
+        elem <- as.vector(population[subpopulation, ])
    
-      for (server in (1:x$nodes))
+      ## Fixed elem to remove the order value
+      elem <- elem[-length(elem)]
+      
+      for (cla in (1:x$classes))
       {
-        demand <- x$vVisit[cla, server] * (1/(x$vMu[cla, server]))
-        #print(paste("demand: ", demand))        
-        #print("LkV: ")
-        #print(LkV[array(data=c(vaux, server), dim=c(1, x$classes+1))])
-
-        if (x$vType[server] == "D")
-          Wck[cla, server] <- demand
-        else        
-          Wck[cla, server] <- demand * (1 + LkV[array(data=c(vaux, server), dim=c(1, x$classes+1))])
-        
-        #print(paste("Wck[" , cla, ", ", server, "]: ", Wck[cla, server]))
+        vaux <- elem
+        vaux[cla] <- max(elem[cla]-1, 1)
+   
+        for (server in (1:x$nodes))
+        {
+          demand <- x$vVisit[cla, server] * (1/(x$vMu[cla, server]))
 
+          if (x$vType[server] == "D")
+            Wck[cla, server] <- demand
+          else        
+            Wck[cla, server] <- demand * (1 + LkV[array(data=c(vaux, server), dim=c(1, x$classes+1))])
+        }
       }
-    }
 
-    for (cla in (1:x$classes))
-    {
-      #print("Throughputc -- calculation")
-      #print(paste("nclass ", cla, " :", elem[cla]-1))
-      #print(paste("denominator: ", x$vThink[cla] + sum(Wck[cla, ])))
-      Throughputc[cla] <- (elem[cla]-1) / (x$vThink[cla] + sum(Wck[cla, ]))
-      #print(paste("Throughputc[", cla, "]: "))
-      #print(Throughputc[cla])
-    }    
+      for (cla in (1:x$classes))
+      {
+        Throughputc[cla] <- (elem[cla]-1) / (x$vThink[cla] + sum(Wck[cla, ]))
+        Throughputcn[array(data=c(elem, cla), dim=c(1, x$classes+1))] <- Throughputc[cla]
+      }
  
-    for (server in (1:x$nodes))
-    {
-      LkV[array(data=c(elem, server), dim=c(1, x$classes+1))] <- sum(Throughputc * Wck[, server])
-      #print("Actualizacion Qk(n)")
-      #print(Throughputc * Wck[, server])
-      #print(sum(Throughputc * Wck[, server]))
-      #print(c(elem, server))
-      #print(array(data=c(elem, server), dim=c(1, x$classes+1)))
-      #print(LkV[array(data=c(elem, server), dim=c(1, x$classes+1))])
+      for (server in (1:x$nodes))
+        LkV[array(data=c(elem, server), dim=c(1, x$classes+1))] <- sum(Throughputc * Wck[, server])
     }
   }
 
@@ -2945,13 +3157,12 @@ QueueingModel.i_MCCN <- function(x, ...)
   {
     ThAux <- Throughputc * x$vVisit[, i]
     Throughputck[ , i] <- ThAux
-    ROck[, i] <- ThAux * x$vMu[, i]
+    ROck[, i] <- ThAux * (1/x$vMu[, i])
     Lck[, i] <- Throughputc * Wck[, i]
   }
 
   Throughput <- 0
   L <- 0
-  W <- 0
 
   for (i in (1:x$nodes))
   {
@@ -2960,12 +3171,12 @@ QueueingModel.i_MCCN <- function(x, ...)
     L <- L + Lk[i]
     Throughputk[i] <- sum(Throughputck[, i])
     Throughput <- Throughput + Throughputk[i]
-    Wk[i] <- Wck[, i] * Throughputc
+    Wk[i] <- sum(Wck[, i] * Throughputc)
   }
   
   Wk <- Wk / Throughput 
 
-  L <- (sum(Wc * Throughputc)) / Throughput
+  W <- (sum(Wc * Throughputc)) / Throughput
 
   res <-
     list(
@@ -2983,7 +3194,8 @@ QueueingModel.i_MCCN <- function(x, ...)
       ROck=ROck,
       Wck=Wck,
       Throughputck=Throughputck,     
-      Lck=Lck
+      Lck=Lck,
+      Throughputcn=Throughputcn
     )
 
   class(res) <- "o_MCCN"
@@ -2991,7 +3203,6 @@ QueueingModel.i_MCCN <- function(x, ...)
 
 }
 
-
 Inputs.o_MCCN <- function(x, ...) { x$Inputs }
 L.o_MCCN <- function(x, ...) { x$L }
 W.o_MCCN <- function(x, ...) { x$W }
@@ -3007,7 +3218,7 @@ ROck.o_MCCN <- function(x, ...) { x$ROck }
 Lck.o_MCCN <- function(x, ...) { x$Lck }
 Wck.o_MCCN <- function(x, ...) { x$Wck }
 Throughputck.o_MCCN <- function(x, ...) { x$Throughputck }
-
+Throughputcn.o_MCCN <- function(x, ...) { x$Throughputcn }
 
 summary.o_MCCN <- function(object, ...)
 {   
@@ -3025,52 +3236,50 @@ summaryMultiClass <- function(object)
   else
     netType <- "closed"
 
-  print(paste("The inputs of the multiclass ", netType, " network are:", sep=""))
+  cat(paste("The inputs of the multiclass ", netType, " network are:", "\n", sep=""))
   print(object$Inputs)
-  print("", quote=FALSE)
-  print(paste("The outputs of the multiclass ", netType, " network are:", sep=""))
-  print("", quote=FALSE)
+  cat("\n\n")
+  cat(paste("The outputs of the multiclass ", netType, " network are:", sep=""))
+  cat("\n\n")
 
-  print("---------- Complete network -------------------------")
-  print(paste("The mean number of clients in the network is: ", object$L))
-  print(paste("The mean time spend in the network is: ", object$W))
-  print(paste("The throughput of the network is: ", object$Throughput))
-  print("", quote=FALSE)
+  cat("---------- Complete network -------------------------\n\n")
+  cat(paste("The mean number of clients in the network is: ", object$L, "\n", sep=""))
+  cat(paste("The mean time spend in the network is: ", object$W, "\n", sep=""))
+  cat(paste("The throughput of the network is: ", object$Throughput, "\n", sep=""))
+  cat("\n\n")
   
-  print("---------- Per Class -------------------------")
+  cat("---------- Per Class -------------------------\n\n")
 
   for (i in (1:object$Inputs$classes))
   {
-    print(paste("The mean number of class ", i, " clients in the network is: ", object$Lc[i], sep=""))
-    print(paste("The mean time spend in the network per class ", i ," is: ", object$Wc[i], sep=""))
-    print(paste("The throughput of class " , i ," of the network is: ", object$Throughputc[i], sep=""))
-    print("", quote=FALSE)    
+    cat(paste("The mean number of class ", i, " clients in the network is: ", object$Lc[i], "\n", sep=""))
+    cat(paste("The mean time spend in the network per class ", i ," is: ", object$Wc[i], "\n", sep=""))
+    cat(paste("The throughput of class " , i ," of the network is: ", object$Throughputc[i], "\n", sep=""))
+    cat("\n\n")    
   }
   
-  print("--------- Per node ---------------------------------")
+  cat("--------- Per node ---------------------------------\n\n")
   
   for (i in (1:object$Inputs$nodes))
   {
-    print(paste("The use of node ", i, " is: ", object$ROk[i], sep=""))
-    print(paste("The mean number of clients in node ", i, " is: ", object$Lk[i], sep=""))
-    print(paste("The mean time spend in node ", i, " is: ", object$Wk[i], sep=""))
-    print(paste("The throughput of node ", i, " is: ", object$Throughputk[i], sep=""))
-    print("", quote=FALSE)
+    cat(paste("The use of node ", i, " is: ", object$ROk[i], "\n", sep=""))
+    cat(paste("The mean number of clients in node ", i, " is: ", object$Lk[i], "\n", sep=""))
+    cat(paste("The mean time spend in node ", i, " is: ", object$Wk[i], "\n", sep=""))
+    cat(paste("The throughput of node ", i, " is: ", object$Throughputk[i], "\n", sep=""))
+    cat("\n\n")
   }
 
-  print("--------- Per class and node -----------------------")
+  cat("--------- Per class and node -----------------------\n\n")
 
   for (i in (1:object$Inputs$classes))
   {
     for (j in (1:object$Inputs$nodes))
     {
-      print(paste("The class ", i, " use of node ", j, " is: ", object$ROck[i, j], sep=""))
-      print(paste("The mean number of class ", i, " clients in node ", j, " is: ", object$Lck[i, j], sep=""))
-      print(paste("The mean time spend by class ", i, " in node ", j, " is: ", object$Wck[i, j], sep=""))
-      print(paste("The throughput of class " , i, " in node ", j, " is: ", object$Throughputck[i, j], sep=""))
-      print("", quote=FALSE)
+      cat(paste("The class ", i, " use of node ", j, " is: ", object$ROck[i, j], "\n", sep=""))
+      cat(paste("The mean number of class ", i, " clients in node ", j, " is: ", object$Lck[i, j], "\n", sep=""))
+      cat(paste("The mean time spend by class ", i, " in node ", j, " is: ", object$Wck[i, j], "\n", sep=""))
+      cat(paste("The throughput of class " , i, " in node ", j, " is: ", object$Throughputck[i, j], "\n", sep=""))
+      cat("\n\n")
     }
   }  
 }
-
-
